{Unit zum Hauptformular}
unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type

{Die Klasse des Hauptfensters (also der Variablentyp)}
  TMainForm = class(TForm)
{von Delphi erzeugter Code beim Hinzufgen der einzelnen Elemente}
    NameListBox: TListBox;
    AddressMemo: TMemo;
    ListLabel: TLabel;
    AddressLabel: TLabel;
    SearchEdit: TEdit;
    AddButton: TButton;
    EditButton: TButton;
    DeleteButton: TButton;
    ExitButton: TButton;
    SearchLabel: TLabel;
    SaveButton: TButton;
    LoadButton: TButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
{die genutzten Ereignisprozeduren, die bei entsprechenden Ereignissen des
Hauptformulars ausgefhrt werden}
    procedure ExitButtonClick(Sender: TObject);
    procedure AddButtonClick(Sender: TObject);
    procedure EditButtonClick(Sender: TObject);
    procedure DeleteButtonClick(Sender: TObject);
    procedure NameListBoxKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure NameListBoxMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure SearchEditChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SaveButtonClick(Sender: TObject);
    procedure LoadButtonClick(Sender: TObject);
  private
    function EntrySelected:Boolean; {True, wenn ein gltiger Eintrag markiert}
    procedure UpdateMemo; {zeigt die Adresse zum ausgewhlten Listeneintrag an}
  public
    { Public-Deklarationen}
  end;

var
  MainForm: TMainForm; {Referenz des Hauptformulars, die Variable}

implementation

{die selbstgeschriebenen Units mssen explizit eingebunden werden,
wenn sie bentigt werden}
uses Edit,Address,Math;

{$R *.DFM}

{Beendet das Programm beim Aktivieren des Beenden-Knopfs}
procedure TMainForm.ExitButtonClick(Sender: TObject);
begin
  Application.Terminate;
end;

{der Hinzufgen-Knopf wurde bettigt:}
procedure TMainForm.AddButtonClick(Sender: TObject);
var Address:TAddress;
begin
{EditForm ist eine Variable aus der Unit Edit fr das Eintrag-Bearbeiten-Fenster
EditForm.Add gibt eine Variable vom Typ TModalResult zurck (siehe Hilfe,und Unit Edit)
bei mrOk wurde der Ok-Knopf bettigt, nur dann soll wirklich hinzugefgt werden}
  if EditForm.Add=mrOk then begin
    Address:=EditForm.getAddress; {den neuen Eintrag zurckgeben lassen}
{den Eintrag in die ListBox hngen:
mit jedem Eintrag kann ein Objekt verknpft werden, wir setzen die anzuzei-
gende Zeile zusammen (Name, Vorname), und hinterlegen den eigentlichen
Address-Eintrag als verknpftes Objekt}
    NameListBox.Items.AddObject(Address.getName+', '+Address.getFirstname,Address);
{den Text des Memos anpassen (evtl. ist jetzt ein anderer Eintrag markiert)}
    UpdateMemo;
  end;
end;

{Der Bearbeiten-Knopf wurde ausgelst}
procedure TMainForm.EditButtonClick(Sender: TObject);
var Address:TAddress;
begin
  if EntrySelected then begin {wenn gltiger Eintrag markiert:}
{den Address-Eintrag als der ListBox holen}
    Address:=TAddress(NameListBox.Items.Objects[NameListBox.ItemIndex]);
{wenn EditForm.Edit mrOk liefert, wurde der Ok-Knopf bettigt
die Addresse mu bergeben werden, damit im Bearbeiten-Fenster die alten Daten
als Vorlage bernommen werden knnen}
    if EditForm.Edit(Address)=mrOk then begin
      Address.Free; {altes Address-Objekt freigeben}
      Address:=EditForm.getAddress; {neues verndertes Objekt zurckgeben lassen}
{den anzuzeigenden String der ListBox ersezten}
      NameListBox.Items.Strings[NameListBox.ItemIndex]:=Address.getName+', '
      +Address.getFirstname;
{das neue Objekt anstelle des alten (inzwischen freigegebenen) Objekts eintragen}
      NameListBox.Items.Objects[NameListBox.ItemIndex]:=Address;
{Memotext anpassen (also die vernderte Adresse anzeigen)}
      UpdateMemo;
    end;
  end;
end;

{der Eintrag-Lschen-Knopf ausgelst:}
procedure TMainForm.DeleteButtonClick(Sender: TObject);
begin
  if EntrySelected then begin {wenn gltiger Eintrag markiert:}
{das mit dem Eintrag verknpfte Address-Objekt freigeben}
    TAddress(NameListBox.Items.Objects[NameListBox.ItemIndex]).Free;
{den Eintrag aus der Listbox lschen}
    NameListBox.Items.Delete(NameListBox.ItemIndex);
{Memotext anpassen}
    UpdateMemo;
  end;
end;

{gibt True zurck, wenn ein Eintrag ausgewhlt wurde}
function TMainForm.EntrySelected:Boolean;
begin
{ListBox.ItemIndex ist die Position des gerade ausgewhlten ListBox-Eintrags}
  EntrySelected:=(NameListBox.ItemIndex>=0) and
  (NameListBox.ItemIndex<NameListBox.Items.Count);
end;

{Wenn eine Taste (der Tastatur) bei Eingaben fr die NameListBox losgelassen
wurde:}
procedure TMainForm.NameListBoxKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
{den Memo-Text anpassen, es knnte jetzt ein anderer Eintrag ausgewhlt sein}
  UpdateMemo;
end;

{Wenn eine Maustaste ber der NameListBox losgelassen wurde:}
procedure TMainForm.NameListBoxMouseUp(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
{den Memo-Text anpassen, es knnte jetzt ein anderer Eintrag ausgewhlt sein}
  UpdateMemo;
end;

{den aktuellen Memo-Text an den ausgewhlten Eintrag anpassen}
procedure TMainForm.UpdateMemo;
var Address:TAddress;
begin
  if EntrySelected then begin {wenn ein gltiger Eingrag ausgewhlt:}
{das Address-Pbjekt aus der NameListBox holen}
    Address:=TAddress(NameListBox.Items.Objects[NameListBox.ItemIndex]);
{den Address-Text des Eintrags anzeigen}
    AddressMemo.Text:=Address.getAddress;
{oben erst Leerzeile und dann "Namen, Vorname" einfgen}
    AddressMemo.Lines.Insert(0,'');
    AddressMemo.Lines.Insert(0,Address.getName+', '+Address.getFirstname);
  end else AddressMemo.Text:='kein Eintrag ausgewhlt';
end;

{wenn sich der Text im SearchEdit (der Eingabe zur Namenssuche) verndert haben
knnte}
procedure TMainForm.SearchEditChange(Sender: TObject);
var Name,Compare:string;
    EqualChars,Index,Lv1,Lv2:Integer;
begin
{wir suchen nach dem Eintrag der Liste, dessen Anfang am weitesten mit der
Such-Eingabe bereinstimmt}
  Name:=LowerCase(SearchEdit.Text); {Sucheingabe}
  EqualChars:=0; {Anzahl der gleichen Buchstaben am Anfang}
  Index:=-1; {Eintrag mit der maximalen bereinstimmung}
{wir gehen alle Eintrge der NameListBox (also alle Addresseintrge) durch}
  for Lv1:=0 to NameListBox.Items.Count-1 do begin
{den Vergleichs-Listeneintrag holen}
    Compare:=LowerCase(NameListBox.Items.Strings[Lv1]);
{Buchstabenweiser Vergleich:}
    for Lv2:=1 to Min(Length(Name),Length(Compare)) do begin
{Abbruch der for-Schleife bei der ersten unterschiedlichen Stelle}
      if Name[Lv2]<>Compare[Lv2] then break;
    end;
{Lv2-1 Buchstaben haben die Beiden Zeichenketten gemeinsam,
wenn das mehr als das bisher gefundene Maximum: neuen Eintrag vormerken,
wenn es weniger sind abbrechen (die Liste ist alphabetisch, wenn sich die
bereinstimmung verringert, ist der beste Eintrag bereits gefunden}
    if Lv2-1>EqualChars then begin
      Index:=Lv1;
      EqualChars:=Lv2-1;
    end else if Lv2-1<EqualChars then break;
  end;

{wenn berhaupt irgendein Eintrag gefunden wurde (eigentlich wird immer einer
gefunden wenn die Liste nicht leer ist)}
  if Index>=0 then begin
    NameListBox.ItemIndex:=Index; {die neue NameListBox-Position setzen}
    UpdateMemo; {Memotext dem neu-ausgewhlten Eintrag anpassen}
  end;
end;

{wenn das Hauptfenster erschaffen wird:}
procedure TMainForm.FormCreate(Sender: TObject);
begin
  UpdateMemo; {sofort den Memotext anpassen}
end;

{wenn das Hauptfenster beseitigt wird:}
procedure TMainForm.FormDestroy(Sender: TObject);
var Lv1:Integer;
begin
{alle Address-Eintrge freigeben}
  for Lv1:=0 to NameListBox.Items.Count-1 do NameListBox.Items.Objects[Lv1].Free;
end;

{Sichert das Adrebuch nach Bettigen von 'Adrebuch Sichern'}
procedure TMainForm.SaveButtonClick(Sender: TObject);
var F:File; {die benutzte untypisierte Dateivariable}
    Lv:Integer;
    Address:TAddress;
begin
{SaveDialog.Execute fhrt einen Windows-blichen Speicherdialog aus und gibt
True zurck, wenn am Ende Ok bettigt wurde}
  if SaveDialog.Execute then begin
    {Dateivariable der ausgewhlten Datei zuordnen}
    AssignFile(F,SaveDialog.FileName);
    {Datei fr Schreibzugriffe ffnen, der bisherige Inhalt wird dabei gelscht
    die 1 gibt an, das in 1-Byte-Einheiten geschrieben wird, d.h smtliche
    Datengren werden in Byte angegeben (Standard ist unverstndlicherweise 128)}
    ReWrite(F,1);
    {Schleife zum Durchgehen aller Adre-Eintrge}
    for Lv:=0 to NameListBox.Items.Count-1 do begin
      {Adresse aus der NameListBox holen}
      Address:=TAddress(NameListBox.Items.Objects[Lv]);
      {Adresse in die Datei schreiben, siehe Unit Address}
      Address.WriteToFile(F);
    end;
    CloseFile(F); {Datei am Ende schlieen}
  end;
end;

{ffnet ein Adrebuch, bei Druck des entsprechenden Knopfes}
procedure TMainForm.LoadButtonClick(Sender: TObject);
var F:File; {untypisierte Dateivariable}
    Address:TAddress;
    Lv:Integer;
begin
{OpenDialog.Execute fhrt einen Windows-blichen ffnendialog aus, und gibt
True zurck, wenn eine Datei ausgewhlt wurde}
  if OpenDialog.Execute then begin
    {alle alten Adre-Eintrge aus der Liste holen und freigeben}
    for Lv:=0 to NameListBox.Items.Count-1 do NameListBox.Items.Objects[Lv].Free;
    {die alte Liste lschen}
    NameListBox.Clear;

    {Die Dateivariable der ausgewhlten Datei zuordnen}
    AssignFile(F,OpenDialog.FileName);
    {Die Datei fr Lesezugriffe ffnen (Dateiinhalt bleibt im Gegensatz zu
    Rewrite erhalten), die 1 zeigt an da alle Datengren in Byte angegeben
    werden}
    Reset(F,1);
    {solange noch ungelesene Daten in der Datei sind:}
    while not EoF(F) do begin
      Address:=TAddress.Create; {Address-Objekt erzeugen}
      Address.ReadFromFile(F); {Addresse aus der Datei lesen, siehe Unit Address}
      {die Adresse wieder in die NameListBox schreiben}
      NameListBox.Items.AddObject(Address.getName+', '+Address.getFirstname,Address);
    end;
    CloseFile(F); {Datei schlieen}

    {wenn die neue Liste Eintrge enthlt, Position innerhalb der Liste auf
    den Anfang setzen}
    if NameListBox.Items.Count>0 then NameListBox.ItemIndex:=0;
    UpdateMemo; {Memotext anpassen}
  end;
end;

end.
